extend("fiskheroes:hero_basic");
loadTextures({
    "layer1": "gowhp:kratos/kratos_layer1",
    "layer2": "gowhp:kratos/kratos_layer2",
    "eyes": "gowhp:kratos/kratos_spartanrage_eyes",
    "shield": "gowhp:kratos/guardianshield",
    "shield_lights": "gowhp:kratos/guardianshield_lights",
    "leviathanaxe": "gowhp:kratos/leviathanaxe",
    "leviathanaxe_lights": "gowhp:kratos/leviathanaxe_lights",
    "mimir": "gowhp:kratos/mimir",
    "mimir_lights": "gowhp:kratos/mimir_lights"
});

var utils = implement("fiskheroes:external/utils");
var flames = implement("gowhp:external/spartanrage");

var leviathanaxe;
var leviathanaxeholster;
var hand_flames;

function init(renderer) {
    parent.init(renderer);
    renderer.setLights((entity) => entity.getData("gowhp:dyn/spartanrage") ? "eyes" : null);
    initEffects(renderer);
}

function initAnimations(renderer) {
    parent.initAnimations(renderer);
    renderer.removeCustomAnimation("basic.BLOCKING");
    renderer.removeCustomAnimation("basic.CHARGED_BEAM");
    renderer.removeCustomAnimation("basic.ENERGY_PROJ");
    addAnimationWithData(renderer, "kratos.BLOCKING", "gowhp:guardianshield", "fiskheroes:shield_blocking_timer")
        .priority = -5;
    addAnimation(renderer, "kratos.DISPLAY", "gowhp:wrathofthefrostancient")
        .setData((entity, data) => data.load(1.0))
        .setCondition(entity => (entity.as("DISPLAY").getDisplayType() == "HOLOGRAM"))
        .priority = 20;
    addAnimationWithData(renderer, "kratos.LEVIATHANAXETHROW", "gowhp:throwaiming", "fiskheroes:beam_charge").setCondition(entity => (entity.getData("gowhp:dyn/leviathanaxe") && entity.getData("gowhp:dyn/leviathanaxe_runic_cycle") == 0));
    addAnimationWithData(renderer, "kratos.WRATHOFTHEFROSTANCIENT", "gowhp:wrathofthefrostancient", "fiskheroes:beam_charge").setCondition(entity => (entity.getData("gowhp:dyn/leviathanaxe") && entity.getData("gowhp:dyn/leviathanaxe_runic_cycle") == 1));
    addAnimationWithData(renderer, "kratos.BREATHOFTHAMUR", "gowhp:breathofthamur", "fiskheroes:beam_charge").setCondition(entity => (entity.getData("gowhp:dyn/leviathanaxe") && entity.getData("gowhp:dyn/leviathanaxe_runic_cycle") == 2));
    addAnimationWithData(renderer, "kratos.LEVIATHANAXEUNHOLSTER", "gowhp:leviathanaxeunholster", "gowhp:dyn/leviathanaxe_timer");
    addAnimationWithData(renderer, "kratos.LEVIATHANAXEFIRSTPERSON", "gowhp:weaponfirstperson", "gowhp:dyn/leviathanaxe").setCondition(entity => (!entity.getInterpolatedData("gowhp:dyn/leviathanaxethrowanimation_timer") && entity.getInterpolatedData("gowhp:dyn/leviathanaxe_timer") > 0.5 && !entity.isPunching()));
    addAnimationWithData(renderer, "kratos.SPARTANRAGECHARGE", "gowhp:spartanragecharge", "gowhp:dyn/spartanrage_timer").setCondition(entity => (entity.getInterpolatedData("gowhp:dyn/spartanrage_timer") > 0.0 && entity.getInterpolatedData("gowhp:dyn/spartanrage_timer") < 0.5));
    addAnimationWithData(renderer, "kratos.SPARTANRAGE", "gowhp:spartanrage", "gowhp:dyn/spartanrage_timer").setCondition(entity => (entity.getInterpolatedData("gowhp:dyn/spartanrage_timer") > 0.5 && entity.getInterpolatedData("gowhp:dyn/spartanrage_timer") < 1.0));
    addAnimationWithData(renderer, "kratos.SPARTANRAGEDUALWIELD", "gowhp:dualwield", "gowhp:dyn/spartanrage").setCondition(entity => entity.isPunching());
}

function initEffects(renderer) {
    shield = renderer.createEffect("fiskheroes:shield");
    shield.texture.set("shield", "shield_lights");
    shield.anchor.set("leftArm");
    shield.setRotation(0.0, 180.0, -10.0).setCurve(25.0, 25.0);
    shield.large = true;

    shieldfp = renderer.createEffect("fiskheroes:shield");
    shieldfp.texture.set("shield", "shield_lights");
    shieldfp.anchor.set("rightArm");
    shieldfp.setRotation(-15.0, 180.0, -45.0).setCurve(25.0, 25.0);
    shieldfp.large = true;

    shieldfpweaponfp = renderer.createEffect("fiskheroes:shield");
    shieldfpweaponfp.texture.set("shield", "shield_lights");
    shieldfpweaponfp.anchor.set("rightArm");
    shieldfpweaponfp.setRotation(-15.0, 125.0, -45.0).setCurve(25.0, 25.0);
    shieldfpweaponfp.large = true;

    var leviathanaxethrow = renderer.createResource("MODEL", "gowhp:leviathanaxe");
    leviathanaxethrow.texture.set("leviathanaxe", "leviathanaxe_lights");
    leviathanaxethrow.bindAnimation("gowhp:leviathanaxethrow").setData((entity, data) => {
		data.load(entity.getInterpolatedData("gowhp:dyn/leviathanaxethrowanimation_timer"));
	});
    leviathanaxe = renderer.createEffect("fiskheroes:model").setModel(leviathanaxethrow);
    leviathanaxe.anchor.set("rightArm");

    utils.bindBeam(renderer, "fiskheroes:charged_beam", "gowhp:invisible", "rightArm", 0x4CB5FF, [
        { "firstPerson": [-4.75, 3.0, -13.0], "offset": [-0.6, 19.0, -1.5], "size": [2.0, 2.0] }
    ]).setCondition(entity => (entity.getData("gowhp:dyn/leviathanaxe") && entity.getData("gowhp:dyn/leviathanaxe_runic_cycle") == 0));

    utils.bindBeam(renderer, "fiskheroes:charged_beam", "gowhp:wrathofthefrostancient", "rightArm", 0x4CB5FF, [
        { "firstPerson": [-6.5, -2.5, -9.0], "offset": [-0.5, 8.0, -14.0], "size": [2.0, 2.0] }
    ]).setCondition(entity => (entity.getData("gowhp:dyn/leviathanaxe") && entity.getData("gowhp:dyn/leviathanaxe_runic_cycle") == 1)).setParticles(renderer.createResource("PARTICLE_EMITTER", "gowhp:wrathofthefrostancient"));;

    utils.bindBeam(renderer, "fiskheroes:charged_beam", "gowhp:breathofthamur", "body", 0x4CB5FF, [
        { "firstPerson": [0.0, 6.0, 0.0], "offset": [0.0, 5.0, -4.0], "size": [40.0, 40.0] }
    ]).setCondition(entity => (entity.getData("gowhp:dyn/leviathanaxe") && entity.getData("gowhp:dyn/leviathanaxe_runic_cycle") == 2)).setParticles(renderer.createResource("PARTICLE_EMITTER", "gowhp:breathofthamur"));;

    var leviathanaxerunicchargeshake = renderer.bindProperty("fiskheroes:camera_shake").setCondition(entity => ((entity.getData("gowhp:dyn/leviathanaxe_runic_cycle") == 1 || entity.getData("gowhp:dyn/leviathanaxe_runic_cycle") == 2) && entity.getData("fiskheroes:beam_charging") && !entity.getData("fiskheroes:beam_shooting")));
    leviathanaxerunicchargeshake.factor = 0.1;

    var leviathanaxerunicshake = renderer.bindProperty("fiskheroes:camera_shake").setCondition(entity => ((entity.getData("gowhp:dyn/leviathanaxe_runic_cycle") == 1 || entity.getData("gowhp:dyn/leviathanaxe_runic_cycle") == 2) && entity.getData("fiskheroes:beam_charging") && entity.getInterpolatedData("fiskheroes:beam_shooting_timer") > 0));
    leviathanaxerunicshake.factor = 0.2;

    leviathanaxeholster = renderer.createEffect("fiskheroes:model");
    leviathanaxeholster.setModel(utils.createModel(renderer, "gowhp:leviathanaxeholster", "leviathanaxe", "leviathanaxe_lights"));
    leviathanaxeholster.anchor.set("body");

    var fire = renderer.createResource("ICON", "fiskheroes:red_fire_layer_%s");
    hand_flames = flames.createHands(renderer, fire, true);

    var spartanrageenergy_color = 0xFF0000;
    var spartanrageenergy_beam = renderer.createResource("BEAM_RENDERER", "gowhp:spartanrageenergy");

    spartanrageenergyarms = utils.createLines(renderer, spartanrageenergy_beam, spartanrageenergy_color, [
        {"start": [-0.3, -1.0, -1.1], "end": [-1.7, 4.5, -1.1], "size": [10.0, 10.0]},
        {"start": [-1.7, -1.0, -1.1], "end": [-0.3, 4.5, -1.1], "size": [10.0, 10.0]},
        {"start": [-1.7, 4.0, -1.1], "end": [-0.3, 6.5, -1.1], "size": [10.0, 10.0]},
        {"start": [-0.3, 4.0, -1.1], "end": [-1.7, 6.5, -1.1], "size": [10.0, 10.0]},

        {"start": [-0.3, -1.0, 1.1], "end": [-1.7, 4.5, 1.1], "size": [10.0, 10.0]},
        {"start": [-1.7, -1.0, 1.1], "end": [-0.3, 4.5, 1.1], "size": [10.0, 10.0]},
        {"start": [-1.7, 4.0, 1.1], "end": [-0.3, 6.5, 1.1], "size": [10.0, 10.0]},
        {"start": [-0.3, 4.0, 1.1], "end": [-1.7, 6.5, 1.1], "size": [10.0, 10.0]},

        {"start": [-1.8, -1.0, 1.0], "end": [-1.8, 4.5, -1.0], "size": [10.0, 10.0]},
        {"start": [-1.8, -1.0, -1.0], "end": [-1.8, 4.5, 1.0], "size": [10.0, 10.0]},
        {"start": [-1.8, 4.0, -1.0], "end": [-1.8, 6.5, 1.0], "size": [10.0, 10.0]},
        {"start": [-1.8, 4.0, 1.0], "end": [-1.8, 6.5, -1.0], "size": [10.0, 10.0]},
    ]);
    spartanrageenergyarms.anchor.set("rightArm");
    spartanrageenergyarms.setScale(1.5);
    spartanrageenergyarms.mirror = true;

    spartanrageenergybody = utils.createLines(renderer, spartanrageenergy_beam, spartanrageenergy_color, [
        {"start": [3.5, 0.0, -1.4], "end": [-3.5, 4.5, -1.4], "size": [10.0, 10.0]},
        {"start": [-3.5, 0.0, -1.4], "end": [3.5, 4.5, -1.4], "size": [10.0, 10.0]},
        {"start": [3.5, 0.0, 1.4], "end": [-3.5, 4.5, 1.4], "size": [10.0, 10.0]},
        {"start": [-3.5, 0.0, 1.4], "end": [3.5, 4.5, 1.4], "size": [10.0, 10.0]},
    ]);
    spartanrageenergybody.anchor.set("body");
    spartanrageenergybody.setScale(1.5);
    spartanrageenergybody.mirror = false;

    utils.bindParticles(renderer, "gowhp:spartanragepunch").setCondition(entity => entity.getData("gowhp:dyn/spartanrage") &&  entity.isPunching());

    utils.addCameraShake(renderer, 0.2, 0.2, "gowhp:dyn/spartanrage");

    var wrathofthefrostancientcharge_color = 0x4CB5FF;
    var wrathofthefrostancientcharge_effect = renderer.createResource("BEAM_RENDERER", "gowhp:wrathofthefrostancientcharge");

    wrathofthefrostancientcharge = utils.createLines(renderer, wrathofthefrostancientcharge_effect, wrathofthefrostancientcharge_color, [
        {"start": [-1.0, 6.0, -15.5], "end": [-1.0, 14.0, -15.5], "size": [30.0, 10.5]},
        {"start": [-1.0, 9.0, -13.6], "end": [-1.0, 14.0, -13.6], "size": [30.0, 10.5]},
        {"start": [-1.0, 11.0, -10.8], "end": [-1.0, 14.0, -10.8], "size": [80.0, 10.5]},
    ]);
    wrathofthefrostancientcharge.anchor.set("rightArm");
    wrathofthefrostancientcharge.setScale(1.0);
    wrathofthefrostancientcharge.mirror = false;

    utils.bindParticles(renderer, "gowhp:wrathofthefrostancientcharge").setCondition(entity => (entity.getData("gowhp:dyn/leviathanaxe") && entity.getData("fiskheroes:beam_charging") && entity.getData("fiskheroes:beam_charge") > 0.5 &&  entity.getData("fiskheroes:beam_charge") < 1 && entity.getData("gowhp:dyn/leviathanaxe_runic_cycle") == 1));

    utils.bindParticles(renderer, "gowhp:breathofthamurcharge").setCondition(entity => (entity.getData("gowhp:dyn/leviathanaxe") && entity.getData("fiskheroes:beam_charging") && entity.getData("fiskheroes:beam_charge") > 0.5 &&  entity.getData("fiskheroes:beam_charge") < 1 && entity.getData("gowhp:dyn/leviathanaxe_runic_cycle") == 2));

    mimir = renderer.createEffect("fiskheroes:model");
    mimir.setModel(utils.createModel(renderer, "gowhp:mimir", "mimir", "mimir_lights"));
    mimir.anchor.set("body");
}

function render(entity, renderLayer, isFirstPersonArm) {
    shield.unfold = entity.getInterpolatedData("fiskheroes:shield_timer");
    shield.setOffset(-4.5, 6.0, 0.0);
    shield.render();
    if (isFirstPersonArm && !(entity.getData("gowhp:dyn/leviathanaxe") || entity.getData("gowhp:dyn/draupnirspear"))) {
        shieldfp.unfold = entity.getInterpolatedData("fiskheroes:shield_timer");
        shieldfp.setOffset(-4.0, 0.0, -14.0);
        shieldfp.render();
    }
    if (isFirstPersonArm && (entity.getData("gowhp:dyn/leviathanaxe") || entity.getData("gowhp:dyn/draupnirspear"))) {
        shieldfpweaponfp.unfold = entity.getInterpolatedData("fiskheroes:shield_timer");
        shieldfpweaponfp.setOffset(-14.0, 4.0, -8.0);
        shieldfpweaponfp.render();
    }
    if ((entity.getInterpolatedData("gowhp:dyn/leviathanaxe_timer") > 0.5) && (entity.getHeldItem().isEmpty()) || (entity.as("DISPLAY").getDisplayType() == "HOLOGRAM" && entity.getWornChestplate().suitType() == 'gowhp:kratos')) {
        leviathanaxe.render();
    }
    if ((renderLayer == "CHESTPLATE" && entity.getInterpolatedData("gowhp:dyn/leviathanaxe_timer") < 0.5) && !(entity.as("DISPLAY").getDisplayType() == "HOLOGRAM" && entity.getWornChestplate().suitType() == 'gowhp:kratos')) {
        leviathanaxeholster.render();
    }
    if (entity.getInterpolatedData("gowhp:dyn/spartanrage_timer")) {
        hand_flames.render(entity);
    }
    if (entity.getInterpolatedData("gowhp:dyn/spartanrage_timer")){
        spartanrageenergyarms.progress = Math.max(1 - (1 - entity.getInterpolatedData('gowhp:dyn/spartanrage_timer')) * 1, 0);
        spartanrageenergyarms.opacity = Math.max(1 - (1 - entity.getInterpolatedData('gowhp:dyn/spartanrage_timer')) * 1, 0);
        spartanrageenergyarms.render();
        spartanrageenergybody.progress = Math.max(1 - (1 - entity.getInterpolatedData('gowhp:dyn/spartanrage_timer')) * 1, 0);
        spartanrageenergybody.opacity = Math.max(1 - (1 - entity.getInterpolatedData('gowhp:dyn/spartanrage_timer')) * 1, 0);
        spartanrageenergybody.render();
    }
    if ((entity.getData("gowhp:dyn/leviathanaxe") && entity.getData("fiskheroes:beam_charge") && entity.getData("gowhp:dyn/leviathanaxe_runic_cycle") == 1)){
        wrathofthefrostancientcharge.progress = Math.max(1 - (1 - entity.getInterpolatedData('fiskheroes:beam_charge')) * 1, 0);
        wrathofthefrostancientcharge.opacity = Math.max(1 - (1 - entity.getInterpolatedData('fiskheroes:beam_charge')) * 1, 0);
        wrathofthefrostancientcharge.render();
    }
    if (renderLayer == "CHESTPLATE") {
        mimir.render();
    }
}
