extend("fiskheroes:hero_basic");
loadTextures({
    "layer1": "gowhp:odin/odin_layer1",
    "layer2": "gowhp:odin/odin_layer2",
    "eyes": "gowhp:odin/odin_bifrost_eyes",
    "gungnir": "gowhp:odin/gungnir",
    "book_of_eldr": "gowhp:odin/book_of_eldr",
});

var utils = implement("fiskheroes:external/utils");
var bifrost_aura = implement("gowhp:external/aura")
var bifrost_aura_fly = implement("gowhp:external/aura_fly")

var gungnir;
var gungnirblade;

function init(renderer) {
    parent.init(renderer);
    renderer.setLights((entity) => (entity.getData("fiskheroes:beam_charging") && (entity.getData("gowhp:dyn/gungnir_runic_cycle") == 0  || entity.getData("gowhp:dyn/gungnir_runic_cycle") == 2)) ? "eyes" : null);
    initEffects(renderer);
}

function initAnimations(renderer) {
    parent.initAnimations(renderer);
    addAnimationWithData(renderer, "odin.BLOCKING", "gowhp:elemental_forcefield", "fiskheroes:shield_blocking_timer")
        .priority = -5;
    addAnimation(renderer, "odin.base", "gowhp:weaponfirstperson")
        .setData((entity, data) => data.load(entity.isAlive()))
        .setCondition(entity => (entity.getHeldItem().isEmpty()) && entity.getData("fiskheroes:shield_blocking_timer") == 0)
        .priority = 10;

    renderer.removeCustomAnimation("basic.BLOCKING");
    renderer.removeCustomAnimation("basic.CHARGED_BEAM");
    renderer.removeCustomAnimation("basic.ENERGY_PROJ");
    utils.addFlightAnimationWithLanding(renderer, "odin.FLIGHT", "gowhp:flight/thor.anim.json");
    utils.addHoverAnimation(renderer, "odin.HOVER", "gowhp:flight/idle/thor_idle");
    utils.addAnimationEvent(renderer, "FLIGHT_DIVE", "gowhp:flight/thor_dive");
    addAnimationWithData(renderer, "odin.LAND", "gowhp:flight/thor_landing", "fiskheroes:dyn/superhero_landing_timer")
        .priority = -8;
    addAnimation(renderer, "odin.DISPLAY", "gowhp:leiptra")
        .setData((entity, data) => data.load(1.0))
        .setCondition(entity => (entity.as("DISPLAY").getDisplayType() == "HOLOGRAM"))
        .priority = 20;
    addAnimationWithData(renderer, "odin.LEIPTRA", "gowhp:leiptra", "fiskheroes:beam_charge").setCondition(entity => (entity.getData("gowhp:dyn/gungnir_runic_cycle") == 0));
    addAnimationWithData(renderer, "odin.ELDR", "gowhp:eldr", "fiskheroes:beam_charge").setCondition(entity => (entity.getData("gowhp:dyn/gungnir_runic_cycle") == 1));
    addAnimationWithData(renderer, "odin.GLETTINGSBARACHARGE", "gowhp:glettingsbara_charge", "fiskheroes:beam_charge").setCondition(entity => (entity.getData("gowhp:dyn/gungnirblade") && entity.getData("gowhp:dyn/gungnir_runic_cycle") == 2 && entity.getData("fiskheroes:beam_charging") && !entity.getData("fiskheroes:beam_shooting")));
    addAnimationWithData(renderer, "odin.GLETTINGSBARA", "gowhp:glettingsbara", "fiskheroes:beam_charge").setCondition(entity => (entity.getData("gowhp:dyn/gungnirblade") && entity.getData("gowhp:dyn/gungnir_runic_cycle") == 2 && entity.getData("fiskheroes:beam_shooting")));
    addAnimationWithData(renderer, "odin.GLETTINGSBARADONE", "gowhp:glettingsbara", "fiskheroes:beam_charge").setCondition(entity => (entity.getData("gowhp:dyn/gungnirblade") && entity.getData("gowhp:dyn/gungnir_runic_cycle") == 2 && !entity.getData("fiskheroes:beam_charging")));
}

function initEffects(renderer) {
    var gungnirblade = renderer.createResource("MODEL", "gowhp:gungnir");
    gungnirblade.texture.set("gungnir");
    gungnirblade.bindAnimation("gowhp:gungnirblade").setData((entity, data) => {
		data.load(entity.getInterpolatedData("gowhp:dyn/gungnirblade_timer"));
	});
    gungnir = renderer.createEffect("fiskheroes:model").setModel(gungnirblade);
    gungnir.anchor.set("rightArm");

    var book_of_eldr_open = renderer.createResource("MODEL", "gowhp:book_of_eldr");
    book_of_eldr_open.texture.set("book_of_eldr");
    book_of_eldr_open.bindAnimation("gowhp:book_of_eldr").setData((entity, data) => {
		data.load((entity.getData("gowhp:dyn/gungnir_runic_cycle") == 1 && entity.getInterpolatedData("fiskheroes:beam_charge")));
	});
    book_of_eldr = renderer.createEffect("fiskheroes:model").setModel(book_of_eldr_open);
    book_of_eldr.anchor.set("rightArm");

    utils.bindBeam(renderer, "fiskheroes:charged_beam", "gowhp:leiptra", "body", 0x651adf, [
        { "firstPerson": [-7.0, -0.5, -9.0], "offset": [0.0, 9.0, -25.0], "size": [2.0, 2.0] }
    ]).setCondition(entity => (entity.getData("gowhp:dyn/gungnir_runic_cycle") == 0)).setParticles(renderer.createResource("PARTICLE_EMITTER", "gowhp:leiptra_smoke"));;

    utils.bindBeam(renderer, "fiskheroes:charged_beam", "gowhp:eldr", "rightArm", 0xe25822, [
        { "firstPerson": [-3.75, 3.0, -6.0], "offset": [-0.5, 14.0, 0.0], "size": [1.5, 1.5] }
    ]).setCondition(entity => (entity.getData("gowhp:dyn/gungnir_runic_cycle") == 1)).setParticles(renderer.createResource("PARTICLE_EMITTER", "gowhp:eldr_impact"));;

    utils.bindParticles(renderer, "gowhp:eldr").setCondition(entity => entity.getData("gowhp:dyn/gungnir_runic_cycle") == 1 &&  entity.getData("fiskheroes:beam_charging"));

    utils.bindBeam(renderer, "fiskheroes:charged_beam", "gowhp:glettingsbara", "body", 0x651adf, [
        { "firstPerson": [0.0, 0.0, 0.0], "offset": [0.0, 0.0, 0.0], "size": [50.0, 50.0] }
    ]).setCondition(entity => (entity.getData("gowhp:dyn/gungnir_runic_cycle") == 2)).setParticles(renderer.createResource("PARTICLE_EMITTER", "gowhp:glettingsbara_impact"));;

    utils.bindBeam(renderer, "fiskheroes:lightning_cast", "gowhp:leiptra", "rightArm", 0x651adf, [
        { "firstPerson": [-4.0, 0.5, -7.0], "offset": [-0.5, 5.0, 0.0], "size": [0.75, 0.75] }
    ]);

    var runicchargeshake = renderer.bindProperty("fiskheroes:camera_shake").setCondition(entity => ((entity.getData("gowhp:dyn/gungnir_runic_cycle") == 0 || entity.getData("gowhp:dyn/gungnir_runic_cycle") == 1 || entity.getData("gowhp:dyn/gungnir_runic_cycle") == 2) && entity.getData("fiskheroes:beam_charging") && !entity.getData("fiskheroes:beam_shooting")));
    runicchargeshake.factor = 0.1;

    var runicshake = renderer.bindProperty("fiskheroes:camera_shake").setCondition(entity => ((entity.getData("gowhp:dyn/gungnir_runic_cycle") == 0 || entity.getData("gowhp:dyn/gungnir_runic_cycle") == 1 || entity.getData("gowhp:dyn/gungnir_runic_cycle") == 2) && entity.getData("fiskheroes:beam_charging") && entity.getInterpolatedData("fiskheroes:beam_shooting_timer") > 0));
    runicshake.factor = 0.2;

    var magic = renderer.bindProperty("fiskheroes:spellcasting");
    magic.colorGeneric.set(0x651adf);
    magic.colorEarthCrack.set(0x651adf);
    magic.colorAtmosphere.set(0x651adf);
    magic.colorWhip.set(0x651adf);

    aura = bifrost_aura.init(renderer, 0x651adf);
    aura_fly = bifrost_aura_fly.init(renderer, 0x651adf);

    var bifrost_color = 0x651adf;
    var bifrost_effect = renderer.createResource("BEAM_RENDERER", "gowhp:bifrost");

    leiptrabeams = utils.createLines(renderer, bifrost_effect, bifrost_color, [
        {"start": [0.0, -5.0, 0.0], "end": [0.0, -15.0, -5.0], "size": [30.0, 10.0]},
        {"start": [0.0, -15.0, -5.0], "end": [0.0, -15.0, -15.0], "size": [30.0, 10.0]},
        {"start": [0.0, -15.0, -15.0], "end": [0.0, 10.0, -25.0], "size": [30.0, 10.0]},

        {"start": [-5.0, 10.0, 0.0], "end": [-15.0, 10.0, -5.0], "size": [10.0, 30.0]},
        {"start": [-15.0, 10.0, -5.0], "end": [-15.0, 10.0, -15.0], "size": [10.0, 30.0]},
        {"start": [-15.0, 10.0, -15.0], "end": [-1.0, 10.0, -25.0], "size": [10.0, 30.0]},

        {"start": [5.0, 10.0, 0.0], "end": [15.0, 10.0, -5.0], "size": [10.0, 30.0]},
        {"start": [15.0, 10.0, -5.0], "end": [15.0, 10.0, -15.0], "size": [10.0, 30.0]},
        {"start": [15.0, 10.0, -15.0], "end": [1.0, 10.0, -25.0], "size": [10.0, 30.0]},
    ]);
    leiptrabeams.anchor.set("body");
    leiptrabeams.setScale(1.0);
    leiptrabeams.mirror = false;

    gungnirbifrost = utils.createLines(renderer, bifrost_effect, bifrost_color, [
        {"start": [-1.0, 8.5, 18.0], "end": [-1.0, 8.5, -14.5], "size": [15.0, 15.0]},

        {"start": [-1.0, 8.5, -10.0], "end": [-1.8, 8.5, -12.8], "size": [15.0, 15.0]},
        {"start": [-1.0, 8.5, -10.0], "end": [-0.2, 8.5, -12.8], "size": [15.0, 15.0]},

        {"start": [-1.0, 8.5, -12.8], "end": [-1.4, 8.5, -14.0], "size": [15.0, 15.0]},
        {"start": [-1.0, 8.5, -12.8], "end": [-0.6, 8.5, -14.0], "size": [15.0, 15.0]},

        {"start": [-2.8, 8.5, -14.0], "end": [-1.0, 8.5, -14.05], "size": [15.0, 15.0]},
        {"start": [0.8, 8.5, -14.0], "end": [-1.0, 8.5, -14.05], "size": [15.0, 15.0]},
    ]);
    gungnirbifrost.anchor.set("rightArm");
    gungnirbifrost.setScale(1.0);
    gungnirbifrost.mirror = false;

    gungnirbladebifrost = utils.createLines(renderer, bifrost_effect, bifrost_color, [
        {"start": [-1.0, 8.5, -14.5], "end": [-1.0, 8.5, -21.1], "size": [15.0, 15.0]},
    ]);
    gungnirbladebifrost.anchor.set("rightArm");
    gungnirbladebifrost.setScale(1.0);
    gungnirbladebifrost.mirror = false;

    var forcefield1 = renderer.bindProperty("fiskheroes:forcefield");
    forcefield1.color.set(0x651adf);
    forcefield1.setShape(30, 15).setOffset(0.0, 6.0, 0.0).setScale(1.55);
    forcefield1.setCondition(entity => {
        forcefield1.opacity = entity.getInterpolatedData("fiskheroes:shield_blocking_timer") * 0.1;
        return true;
    });

    var forcefield2 = renderer.bindProperty("fiskheroes:forcefield");
    forcefield2.color.set(0xe25822);
    forcefield2.setShape(20, 10).setOffset(0.0, 6.0, 0.0).setScale(1.5);
    forcefield2.setCondition(entity => {
        forcefield2.opacity = entity.getInterpolatedData("fiskheroes:shield_blocking_timer") * 0.2;
        return true;
    });

    var forcefield3 = renderer.bindProperty("fiskheroes:forcefield");
    forcefield3.color.set(0x4CB5FF);
    forcefield3.setShape(10, 5).setOffset(0.0, 6.0, 0.0).setScale(1.45);
    forcefield3.setCondition(entity => {
        forcefield3.opacity = entity.getInterpolatedData("fiskheroes:shield_blocking_timer") * 0.3;
        return true;
    });

    utils.bindParticles(renderer, "gowhp:elemental_forcefield").setCondition(entity => entity.getInterpolatedData("fiskheroes:shield_blocking"));

    var leiptraorb = renderer.bindProperty("fiskheroes:forcefield");
    leiptraorb.color.set(0x651adf);
    leiptraorb.setShape(10, 5).setOffset(0.0, 6.0, 0.0).setScale(1.3);
    leiptraorb.setCondition(entity => {
        leiptraorb.opacity = (entity.getData("gowhp:dyn/gungnir_runic_cycle") == 0) && (entity.getInterpolatedData('fiskheroes:beam_charge') * 0.1);
        return true;
    });

    var leiptraorbdisplay = renderer.bindProperty("fiskheroes:forcefield");
    leiptraorbdisplay.color.set(0x651adf);
    leiptraorbdisplay.setShape(10, 5).setOffset(0.0, 6.0, 0.0).setScale(1.3);
    leiptraorbdisplay.setCondition(entity => {
        leiptraorbdisplay.opacity = 0.1;
        return entity.as("DISPLAY").getDisplayType() == "HOLOGRAM";
    });

    var eldrorb = renderer.bindProperty("fiskheroes:forcefield");
    eldrorb.color.set(0xe25822);
    eldrorb.setShape(60, 30).setOffset(0.0, 6.0, 0.0).setScale(1.3);
    eldrorb.setCondition(entity => {
        eldrorb.opacity = (entity.getData("gowhp:dyn/gungnir_runic_cycle") == 1) && (entity.getInterpolatedData('fiskheroes:beam_charge') * 0.2);
        return true;;
    });

    utils.bindParticles(renderer, "gowhp:bifrost_effect").setCondition(entity => (entity.getData("fiskheroes:beam_charging") && (entity.getData("gowhp:dyn/gungnir_runic_cycle") == 0  || entity.getData("gowhp:dyn/gungnir_runic_cycle") == 2)) || (entity.getData("fiskheroes:flying")));

    utils.bindParticles(renderer, "gowhp:glettingsbara").setCondition(entity => entity.getData("gowhp:dyn/gungnir_runic_cycle") == 2 &&  entity.getData("fiskheroes:beam_shooting_timer") > 0);

    utils.bindTrail(renderer, "gowhp:bifrost").setCondition(entity => (entity.getData("fiskheroes:beam_charging") && entity.getData("gowhp:dyn/gungnir_runic_cycle") == 0));
}

function render(entity, renderLayer, isFirstPersonArm) {
    if ((renderLayer == "CHESTPLATE" && entity.getHeldItem().isEmpty()) || (entity.as("DISPLAY").getDisplayType() == "HOLOGRAM")) {
        gungnir.render();
    }
    if (entity.getData("fiskheroes:beam_charge") && entity.getData("gowhp:dyn/gungnir_runic_cycle") == 0){
        leiptrabeams.progress = Math.max(1 - (1 - entity.getInterpolatedData('fiskheroes:beam_charge')) * 1, 0);
        leiptrabeams.opacity = Math.max(1 - (1 - entity.getInterpolatedData('fiskheroes:beam_charge')) * 1, 0);
        leiptrabeams.render();
    }
    if ((entity.as("DISPLAY").getDisplayType() == "HOLOGRAM")){
        leiptrabeams.progress = true;
        leiptrabeams.opacity = true;
        leiptrabeams.render();
    }
    if ((entity.getData("fiskheroes:beam_charge") && (entity.getData("gowhp:dyn/gungnir_runic_cycle") == 0 || entity.getData("gowhp:dyn/gungnir_runic_cycle") == 2))){
        gungnirbifrost.opacity = Math.max(1 - (1 - entity.getInterpolatedData('fiskheroes:beam_charge')) * 1, 0);
        gungnirbifrost.render();
    }
    if (entity.getData("fiskheroes:flying") || entity.getInterpolatedData("fiskheroes:shield_blocking") || (entity.as("DISPLAY").getDisplayType() == "HOLOGRAM")){
        gungnirbifrost.progress = true;
        gungnirbifrost.opacity = true;
        gungnirbifrost.render();
    }
    if (entity.getData("gowhp:dyn/gungnirblade") && (entity.getData("fiskheroes:beam_charge") && (entity.getData("gowhp:dyn/gungnir_runic_cycle") == 0  || entity.getData("gowhp:dyn/gungnir_runic_cycle") == 2))){
        gungnirbladebifrost.opacity = Math.max(1 - (1 - entity.getInterpolatedData('fiskheroes:beam_charge')) * 1, 0);
        gungnirbladebifrost.render();
    }
    if (entity.getData("gowhp:dyn/gungnirblade") && (entity.getData("fiskheroes:flying") || entity.getInterpolatedData("fiskheroes:shield_blocking"))){
        gungnirbladebifrost.progress = Math.max(1 - (1 - entity.getInterpolatedData('gowhp:dyn/gungnirblade_timer')) * 1, 0);
        gungnirbladebifrost.opacity = true;
        gungnirbladebifrost.render();
    }
    if ((entity.getData("fiskheroes:beam_charge") && (entity.getData("gowhp:dyn/gungnir_runic_cycle") == 0 || entity.getData("gowhp:dyn/gungnir_runic_cycle") == 2)) || (entity.as("DISPLAY").getDisplayType() == "HOLOGRAM")){
        aura.render(entity, renderLayer);
    }
    if (entity.getData("fiskheroes:flying")){
        aura_fly.render(entity, renderLayer);
    }
    if ((entity.getData("fiskheroes:beam_charge") && entity.getData("gowhp:dyn/gungnir_runic_cycle") == 1)){
        book_of_eldr.render();
    }
}